/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui.components;

import event.EventData;
import event.events.StringEvent;
import logging.Logger;
import main.GameApp;
import org.lwjgl.input.Keyboard;

/**
 * Ein Textfield kann eine einzelne Zeile Text aufnehmen und kann vom Benutzer
 * veraendert werden, wenn es den Fokus hat.
 * @author nico3000
 */
public class TextField extends Component {
    /**
     * Der aktuelle Text.
     */
    private String text = "";
    /**
     * Die Zeit in Millisekunden seit der letzten Aenderung.
     */
    private long millis = 0;
    /**
     * Gibt an, ob das Textfield den Fokus hat.
     */
    private boolean focused = false;
    /**
     * Die Position des Textcursors.
     */
    private int caretPosition = 0;
    /**
     * Die Schriftgroesse.
     */
    private int fontSize = 12;

    /**
     * Konstruktor.
     * @param text Initialer Text
     */
    public TextField(String text) {
        this.setText(text);
    }

    /**
     * Liefert den Text.
     * @return Der Text
     */
    public String getText() {
        return text;
    }

    /**
     * Setzt einen neuen Text.
     * @param text Der neue Text
     */
    public final void setText(String text) {
        this.text = text;
        this.caretPosition = this.text.length();
    }

    @Override
    public boolean onCursorEntered(int x, int y) {
        return true;
    }

    @Override
    public void onCursorLeft() {
        // nothing to do
    }

    @Override
    public void onCursorMoved(int posX, int posY, int dx, int dy) {
        // nothing to do
    }

    @Override
    public boolean onFocusGained() {
        this.focused = true;
        this.repaint();
        return true;
    }

    @Override
    public void onFocusLost() {
        this.focused = false;
        this.repaint();
    }

    @Override
    public boolean onButtonDown(int button, int x, int y) {
        return true;
    }

    @Override
    public boolean onButtonUp(int button, int x, int y, boolean clicked) {
        return true;
    }

    @Override
    public boolean onDragged(int button, int x, int y, int dx, int dy) {
        return false;
    }

    @Override
    public void onUpdate(long deltaMillis) {
        boolean drawCursor = this.drawCursor();
        this.millis += deltaMillis;
        if(this.drawCursor() != drawCursor) {
            this.repaint();
        }
    }
    
    @Override
    protected void repaint() {
        this.setTexture(this.getGui().getDrawer().createTextField(
                this.text,
                this.drawCursor() ? this.caretPosition : -1,
                this.fontSize,
                this.getWidth(),
                this.getHeight()));
    }

    /**
     * Setzt die Schriftgroesse.
     * @param fontSize Die neue Groesse
     */
    public void setFontSize(int fontSize) {
        this.fontSize = fontSize;
    }
    
    /**
     * Prueft, ob der Textcursor gezeichnet werden soll.
     * @return <tt>true</tt>, wenn der Cursor gezeichnet werden soll;
     *         <tt>false</tt> sonst
     */
    private boolean drawCursor() {
        return this.focused && (this.millis / 400) % 2 == 0;
    }

    @Override
    public boolean onKeyDown(int key, char c) {
        switch(key) {
            case Keyboard.KEY_BACK:
                if(this.caretPosition != 0) {
                    this.text = this.text.substring(0, this.caretPosition - 1) + this.text.substring(this.caretPosition);
                    --this.caretPosition;
                }
                break;
            case Keyboard.KEY_DELETE:
                if(this.caretPosition != this.text.length()) {
                    this.text = this.text.substring(0, this.caretPosition) + this.text.substring(this.caretPosition + 1);
                }
                break;
            case Keyboard.KEY_HOME:
                this.caretPosition = 0;
                break;
            case Keyboard.KEY_END:
                this.caretPosition = this.text.length();
                break;
            case Keyboard.KEY_RIGHT:
                this.caretPosition = Math.min(this.text.length(), this.caretPosition + 1);
                break;
            case Keyboard.KEY_LEFT:
                this.caretPosition = Math.max(0, this.caretPosition - 1);
                break;
            case Keyboard.KEY_RETURN:
                this.fireAction(this.text);
                break;
            default:
                if(!Character.isISOControl(c)) {
                    //Logger.INSTANCE.info(Keyboard.getKeyName(key));
                    //Logger.INSTANCE.info(Character.getName(c))
                    this.text = this.text.substring(0, this.caretPosition) + c + this.text.substring(this.caretPosition);
                    ++this.caretPosition;
                }
                break;
        }
        this.millis = 0;
        this.repaint();
        return true;
    }

    @Override
    public boolean onKeyUp(int key) {
        return false;
    }
    
}
