/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package resource.file;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import logging.Logger;
import resource.Resource;

/**
 * Eine ResourceFile, die Dateien aus einem normalen Ordner bezieht.
 * @author nico3000
 */
public class DirectoryResourceFile implements ResourceFile {
    /**
     * Der Ordner.
     */
    private final String directory;

    /**
     * Konstruktor.
     * @param directory Der Ordner
     */
    public DirectoryResourceFile(String directory) {
        if(directory == null) {
            this.directory = "./";
        } else if(directory.trim().endsWith("/") || directory.trim().endsWith("\\")) {
            this.directory = directory.trim();
        } else {
            this.directory = directory.trim() + "/";
        }
    }

    @Override
    public boolean open() {
        File dir = new File(this.directory);
        if(!dir.exists()) {
            Logger.INSTANCE.error(this.directory + " does not exist");
            return false;
        } else if(!dir.isDirectory()) {
            Logger.INSTANCE.error(this.directory + " is no directory");
            return false;
        } else if(!dir.canRead()) {
            Logger.INSTANCE.error("cannot read from " + this.directory);
            return false;
        }
        return true;
    }

    @Override
    public void close() {
        // nothing to do
    }

    @Override
    public int getRawResourceSize(Resource resource) {
        File file = new File(this.directory + resource.getName());
        return (int)file.length();
    }

    @Override
    public int getRawResource(Resource resource, byte buffer[]) {
        int read = 0;
        FileInputStream stream = null;
        try {
            stream = new FileInputStream(this.directory + resource.getName());
            while(read != buffer.length) {
                read += stream.read(buffer, read, buffer.length - read);        // TODO: may block forever?
            }
        } catch (IOException ex) {
            Logger.INSTANCE.error(ex.getMessage());
        } finally {
            if (stream != null) {
                try {
                    stream.close();
                } catch (IOException ex) {
                    Logger.INSTANCE.error(ex.getMessage());
                }
            }
        }
        return read;
    }

    @Override
    public int getNumResources() {
        return new File(this.directory).list().length;
    }

    @Override
    public String getResourceName(int num) {
        return new File(directory).list()[num];
    }
}
