/*
 * Copyright (C) 2013 nico3000 - All Rights Reserved
 */
package resource.loader;

import resource.ResHandle;

/**
 * Ein ResourceLoader wandelt die Rohdaten einer Resource in weiter benutzbare
 * Daten um.
 * @author nico3000
 */
public interface ResourceLoader {
    /**
     * Das Pattern fuer die Methode <tt>String.match()</tt>. Liefert diese mit
     * dem Namen der Resource <tt>true</tt> zurueck, sollten die Daten mithilfe
     * dieser Klasse geladen werden koennen.
     * @return Das Pattern
     */
    public String getPattern();
    
    /**
     * Liefert, ob die Daten nach dem Laden von dieser Klasse weiterverarbeitet
     * werden.
     * @return <tt>true</tt>, falls die Daten weiterverarbeitet werden 
     */
    public boolean useRawFile();
    
    /**
     * Liefert die Groesse der verarbeiteten Daten anhand der Rohdaten in Byte
     * zurueck.
     * @param buffer Der Buffer, der die Rohdaten enthaelt
     * @param rawSize Die Groesse der Rohdaten in Byte
     * @return Die Groesse der verarbeiteten Daten in Byte
     */
    public int getLoadedResourceSize(byte buffer[], int rawSize);
    
    /**
     * Verarbeitet Rohdaten und speichert sie anschliessend in den Daten eines
     * Handles. Das Handle hat Platz fuer die Anzahl Bytes, die
     * <tt>getLoadedResourceSize()</tt> zurueckgegeben hat.
     * @param rawBuffer Der Buffer, der die Rohdaten enthaelt
     * @param rawSize Die Groesse der Rohdaten in Byte
     * @param handle Der Handle, in den die verarbeiteten Daten geschrieben
     *               werden sollen
     * @return <tt>true</tt>, falls erfolgreich
     */
    public boolean loadResource(byte rawBuffer[], int rawSize, ResHandle handle);
}
