/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package view;

import actor.ActorFactory;

/**
 * Eine GameView stellt eine bestimmte Sicht auf die GameLogic dar und bedient
 * normalerweise die Sinne des Spielers.
 * @author nico3000
 */
public abstract class GameView {
    /**
     * Der mit der View verbundene Actor.
     */
    private int actorId = ActorFactory.INVALID_ACTOR_ID;
    
    /**
     * Liefert ID des mit der View verbundenen Actors.
     * @return Die ID des Actors
     */
    public final int getActorId() {
        return this.actorId;
    }
    
    /**
     * Verknuepft einen Actor mit dieser View.
     * @param id Die ID des Actors
     */
    public final void setActorId(int id) {
        this.actorId = id;
    }
    
    /**
     * Initialisiert die View.
     * @return <tt>true</tt>, falls erfolgreich
     */
    public abstract boolean init();
    
    /**
     * Zerstoert eine View.
     */
    public abstract void destroy();
    
    /**
     * Aktualisiert das Element.
     * @param deltaMillis Die Anzahl der Millieskunden, die seit dem letzten
     *                    Aufruf vergangen ist
     */
    public abstract void onUpdate(long deltaMillis);
    
    /**
     * Rendert die Grafikausgabe.
     */
    public abstract void onRender();
    
    /**
     * Wird aufgerufen, wenn das Graphicsdevice wiederhergestellt und nachdem
     * das Element erfolgreich der <tt>HumanView</tt> hinzugefuegt wurde, um das
     * Element zu re- oder initialisieren.
     * @return <tt>true</tt>, falls erfolgreich
     */
    public abstract boolean onRestore();
    
    /**
     * Wird aufgerufen, wenn das Graphicsdevice verloren geht. <i>Hinweis</i>:
     * Uns ist nicht bekannt, wann das jemals geschieht.
     * @return <tt>true</tt>, wenn erfolgreich
     */
    public abstract boolean onLostDevice();
}
