/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package view;

import controls.FpsControlsProcess;
import controls.MoveEvents;
import event.events.ToggleEvent;
import event.EventData;
import event.EventListener;
import event.EventManager;
import event.events.ActorCreatedEvent;
import event.events.ActorDeletedEvent;
import event.events.PlayerActorChangedEvent;
import logging.Logger;
import main.GameApp;
import org.lwjgl.input.Mouse;

/**
 * Diese Klasse empfaengt Events, die den Spieler bewegen sollen, prueft ob
 * die Bewegung moeglich ist und fuehrt sie anschliessend aus. Momentan sind
 * alle Bewegungen moeglich.
 * @author nico3000
 */
public class PlayerControlsValidator implements EventListener {
    /**
     * Der Prozess, der die Bewegungen letztendlich ausfuehrt.
     */
    private final FpsControlsProcess moveProcess = new FpsControlsProcess();
    
    /**
     * Initialisiert die Instanz.
     * @return <tt>true</tt>, falls erfolgreich; <tt>false</tt> sonst
     */
    public boolean init() {
        GameApp.getGlobalProcessManager().attachProcess(this.moveProcess);
        if(!GameApp.getEventManager().register(this,
                PlayerActorChangedEvent.ID,
                MoveEvents.ForwardEvent.ID,
                MoveEvents.BackwardEvent.ID,
                MoveEvents.LeftEvent.ID,
                MoveEvents.RightEvent.ID,
                MoveEvents.UpEvent.ID,
                MoveEvents.DownEvent.ID,
                MoveEvents.FastEvent.ID,
                MoveEvents.ViewEvent.ID,
                MoveEvents.GrabMouseEvent.ID,
                ActorCreatedEvent.ID,
                ActorDeletedEvent.ID)) {
            Logger.INSTANCE.error("Registering EventListener failed!");
            return false;
        }
        return true;
    }
    
    /**
     * Entfernt die Instanz aus dem System.
     */
    public void destroy() {
        GameApp.getEventManager().unregister(this, EventManager.TYPE_ALL);
        this.moveProcess.succeed();
    }

    @Override
    public void trigger(EventData data) {
        if(data.getId() == PlayerActorChangedEvent.ID) {
            this.moveProcess.setActor(GameApp.getGameLogic().getActor(((PlayerActorChangedEvent)data).getActorId()));
        } else if(data instanceof ToggleEvent) {
            ToggleEvent evt = (ToggleEvent)data;
            switch(evt.getId()) {
                case MoveEvents.ForwardEvent.ID:
                    this.moveProcess.getMoveVector().z += evt.isActivated() ? +1.0f : -1.0f;
                    break;
                case MoveEvents.BackwardEvent.ID:
                    this.moveProcess.getMoveVector().z += evt.isActivated() ? -1.0f : +1.0f;
                    break;
                case MoveEvents.LeftEvent.ID:
                    this.moveProcess.getMoveVector().x += evt.isActivated() ? -1.0f : +1.0f;
                    break;
                case MoveEvents.RightEvent.ID:
                    this.moveProcess.getMoveVector().x += evt.isActivated() ? +1.0f : -1.0f;
                    break;
                case MoveEvents.UpEvent.ID:
                    this.moveProcess.getMoveVector().y += evt.isActivated() ? +1.0f : -1.0f;
                    break;
                case MoveEvents.DownEvent.ID:
                    this.moveProcess.getMoveVector().y += evt.isActivated() ? -1.0f : +1.0f;
                    break;
                case MoveEvents.FastEvent.ID:
                    this.moveProcess.setSpeedModifier(evt.isActivated() ? 4.0f : 1.0f);
                    break;
            }
        } else if(data.getId() == MoveEvents.ViewEvent.ID) {
            MoveEvents.ViewEvent evt = (MoveEvents.ViewEvent)data;
            this.moveProcess.setOrientation(-1e-2f * (float)evt.getDeltaX(), 1e-2f * (float)evt.getDeltaY(), 0.0f);
        } else if(data.getId() == MoveEvents.GrabMouseEvent.ID) {
            Mouse.setGrabbed(!Mouse.isGrabbed());
        }
    }
}
